#ifndef GEMFREADER_H
#define GEMFREADER_H

#include <QtGui>

#include "ReaderInterface.h"

struct GEMFRange
{
	quint32 zoom;
	quint32 xmin;
	quint32 xmax;
	quint32 ymin;
	quint32 ymax;
	quint32 source_index;
	quint64 offset;
};


class GEMFReader : public ReaderInterface
{
public:
	void Initialise(QString location);
	void Close();
	bool IsOpen();
	QImage GetImage(quint32 x_index, quint32 y_index, quint32 zoom);
	QList<quint32> GetZoomLevels();
	QList<quint32> GetZoomLevels(quint32 source_index);
	QHash<quint32,QString> GetSourceList();
	void SelectSource(quint32 source);
	void EnableSource(quint32 source, bool bEnabled=true);
	void DisableSource(quint32 source);
	void AcceptAnySource();
	QList<Coverage> GetMapCoverage(quint32 source_index, quint32 zoom);

private:
	quint32 ArrayToInt(QByteArray array);
	bool ReadUInt(QFile &fh, quint32 &result);
	bool ReadULong(QFile &fh, quint64 &result);
	quint64 ArrayToLong(QByteArray array);
	bool ReadGEMFHeader(QString location);
	Coverage RangeToCoverage(GEMFRange rs, quint32 zoom);

	QString loadedGEMFFile;
	QFile gemf_file;
	QList<GEMFRange> lstRangeData;
	QHash<quint32,QString> hshSources;
	QHash<quint32, bool> hshSourceStates;
	quint32 m_TileSize;
	QList<quint64> lstFileSizes;
	QList<QString> lstFileNames;

};

#endif
