#ifndef MERCATOR_H
#define MERCATOR_H

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

#include <QtGui>

struct LatLon
{
	double latitude;
	double longitude;
};
struct Metres
{
	double mx;
	double my;
};
struct Pixels
{
	double px;
	double py;
};

struct Tile
{
	quint32 indexX;
	quint32 indexY;
	quint32 posX;
	quint32 posY;
};


class Mercator
{
	static const double m_OriginShift = 2 * M_PI * 6378137 / 2.0;

public:
	static LatLon MetresToLatLon(Metres metres);
	static Metres LatLonToMetres(LatLon latlon);
	static double Resolution(quint32 zoom, quint32 tilesize);
	static Pixels MetresToPixels(Metres metres, quint32 zoom, quint32 tilesize);
	static Metres PixelsToMetres(Pixels pixels, quint32 zoom, quint32 tilesize);
	static Tile PixelsToTMSTile(Pixels pixels, quint32 tilesize);
	static Tile PixelsToOSMTile(Pixels pixels, quint32 zoom, quint32 tilesize);
	static Tile TMSTileToOSMTile(Tile t, quint32 zoom);
	static Tile LatLonToOSMTile(LatLon latlon, quint32 zoom, quint32 tilesize);
	static LatLon PixelsToLatLon(Pixels pixels, quint32 zoom, quint32 tilesize);
	static Pixels LatLonToPixels(LatLon latlon, quint32 zoom, quint32 tilesize);
	static QPointF PixelsToWindowPos(Pixels p, LatLon pageCentre, QSize winSize, quint32 zoom, quint32 tilesize);
	static Pixels WindowPosToPixels(QPointF p, LatLon pageCentre, QSize winSize, quint32 zoom, quint32 tilesize);
	static QPointF LatLonToWindowPos(LatLon l, LatLon pageCentre, QSize winSize, quint32 zoom, quint32 tilesize);
	static LatLon WindowPosToLatLon(QPointF p, LatLon pageCentre, QSize winSize, quint32 zoom, quint32 tilesize);
	static QPair<LatLon, LatLon> TMSTileLatLonBounds(Tile t, quint32 zoom, quint32 tilesize);
	static QPair<LatLon, LatLon> OSMTileLatLonBounds(Tile t, quint32 zoom, quint32 tilesize);
	static QPair<Metres, Metres> TMSTileMetreBounds(Tile t, quint32 zoom, quint32 tilesize);
	static Tile OSMTileToTMSTile(Tile t, quint32 zoom);
};

#endif
