#ifndef QMAPVIEWSCENE_H
#define QMAPVIEWSCENE_H

#include <QFrame>
#include <QDir>

#include "Mercator.h"
#include "ReaderInterface.h"

enum Mode
{
	ModePan,
	ModeSelect,
};

class ViewInfo
{
public:
	LatLon PageCentre;
	quint32 ZoomLevel;
	QList<quint32> lstZoomLevels;
	QString MapStore;
};

class QMapViewScene : public QFrame
{
	Q_OBJECT

public:
	QMapViewScene(QWidget *parent = 0);
	void SetTileSize(quint32 tilesize);
	void SetMode(Mode m);
	void SetMapLocation(QString location);
	void SetZoomLevel(quint32 zoom);
	QHash<quint32, QList<quint32> > GetZoomLevelsAndSources(void);
	void ZoomIn();
	void ZoomOut();

public slots:
	void SetScalingAllowed(bool allowed);
	QHash<quint32,QString> GetSourceList();
	void SelectSource(quint32 source);
	void AcceptAnySource();
	void EnableSource(quint32 source, bool bEnabled=true);
	void DisableSource(quint32 source);
	void ClearCoverage(void);
	void ShowCoverage(quint32 source_index, quint32 zoom);

protected:
	void mousePressEvent(QMouseEvent *mouseEvent);
	void mouseMoveEvent(QMouseEvent *mouseEvent);
	void mouseReleaseEvent(QMouseEvent *mouseEvent);
	void paintEvent(QPaintEvent *event);

private:
	Tile tileIndexFromCoordinate(quint32 zoom, double latitude, double longitude);
	ViewInfo m_ViewInfo;
	QList<LatLon> m_lstMarkerLocations;
	QRectF getMarkerBoundingBox(LatLon marker);
	QList<QImage> GetImage(quint32 x_index, quint32 y_index);
	QImage GetImageFromGEMF(quint32 x_index, quint32 y_index, quint32 zoom);
	QImage GetImage(quint32 x_index, quint32 y_index, quint32 zoom);
	QImage GetImageFromMaverickDir(QDir mapDir, quint32 x_index, quint32 y_index, quint32 zoom);
	bool ReadGEMFHeader(QString location);
	Tile FindTileContaining(LatLon l, quint32 zoom);
	void DrawImage(QPainter *painter, QImage img, QPointF corner);
	QList<quint32> GetZoomLevels(QString location);
	quint32 m_TileSize;
	QImage m_BlankImage;
	QPointF m_StartPoint;
	Pixels m_StartCentre;
	bool m_Moving;
	bool m_Selecting;
	bool m_bCanScale;
	Mode m_Mode;
	QRectF m_SelectionRect;
	ReaderInterface *pImgReader;
	QList<Coverage> lstCoverage;

	static const quint32 m_MinZoom = 1;
	static const quint32 m_MaxZoom = 17;
};

#endif
